package flow.stages

import chisel3._
import flow.Params
import flow.components.newProgramCounter
import flow.components.newPcControlInterface
import flow.stages.utils._
import flow.stages.messages._
import chisel3.util.DecoupledIO

class IF(implicit val p: Params) extends Module {
  import flow.components.RV32InstSubfields._

  // Use DPI-C for instant instruction fetch for now
  val io = IO(new Bundle {
    val fromRam = Input(new Ram2IF)
    val toRam = Output(new IF2Ram)

    val fromEx = Input(new EX2IF)
  })

  val msgio = IO(new Bundle {
    val out = DecoupledIO(new IF2ID)
  })
  msgio.out.valid := DontCare
  // val msgio = IO(DecoupledMsgIO(out = (new IF2ID).S))

  val _out = msgio.out.bits
  val _fromRam = io.fromRam
  val _toRam = io.toRam
  val _fromEx = io.fromEx

  // Program Counter
  private val pc = Module(new newProgramCounter)

  // PC update
  pc.in.brOffset := _fromEx.brOffset
  pc.in.jAddr := _fromEx.jAddr

  pc.control := _fromEx.pcCtrl

  // Instruction fetch
  _toRam.pc := pc.out
  _out.inst := _fromRam.inst
  _out.pc := pc.out
}
