package flow.stages

import chisel3._
import chisel3.util.Decoupled
import chisel3.util.DecoupledIO
import flow.Params
import flow.stages.utils._
import flow.stages.messages._
import flow.components.RamDpi

class LS(implicit val p: Params) extends Module {
  val io = IO(new Bundle {
    val toIF = Output(new Ram2IF)
    val fromIF = Input(new IF2Ram)
  })
  // val msgio = IO(DecoupledMsgIO((new EX2LS).S, (new LS2WB).S))
  val msgio = IO(new Bundle {
    val in = Flipped(DecoupledIO(new EX2LS))
    val out = DecoupledIO(new LS2WB)
  })
  msgio.in.ready := DontCare
  msgio.out.valid := DontCare
  private val _in = msgio.in.bits
  private val _out = msgio.out.bits
  private val _toIF = io.toIF
  private val _fromIF = io.fromIF

  val ram = Module(new RamDpi)

  ram.io.clock := clock
  ram.io.reset := reset
  ram.io.writeAddr := _in.exeOut
  ram.io.writeData := _in.src2
  ram.io.writeMask := _in.ramCtrl.writeMask
  ram.io.readMask := _in.ramCtrl.readMask

  ram.io.writeEnable := _in.ramCtrl.writeEnable
  ram.io.valid := _in.ramCtrl.valid // TODO: change to a better name
  ram.io.readAddr := _in.exeOut

  // TODO: Change to icache, and move to IF stage.
  // Change to arbitor here
  ram.io.pc := _fromIF.pc
  _toIF.inst := ram.io.inst

  _out.memOut := ram.io.readData
  _out.exeOut := _in.exeOut

  // passthrough
  _out.regCtrl := _in.regCtrl
  _out.rd := _in.rd
  _out.pc := _in.pc
}
